@@ -3,84 +3,122 @@
 #' @description
 #' This class is a wrapper for a series of beta-diversity related analysis, 
 #' including several ordination calculations and plotting based on An et al. (2019) <doi:10.1016/j.geoderma.2018.09.035>, group distance comparision, 
-#' clustering and perMANOVA based on Anderson al. (2008) <doi:10.1111/j.1442-9993.2001.01070.pp.x>.
+#' clustering, perMANOVA based on Anderson al. (2008) <doi:10.1111/j.1442-9993.2001.01070.pp.x> and PERMDISP.
 #'
 #' @export
 trans_beta <- R6Class(classname = "trans_beta",
 	public = list(
 		#' @param dataset the object of \code{\link{microtable}} Class.
-		#' @param ordination default NULL; PCA, PCoA or NMDS.
-		#' @param measure default NULL; bray, jaccard, wei_unifrac or unwei_unifrac, or other name of matrix you add; beta diversity index used for ordination, manova or group distance.
-		#' @param group default NULL; sample group used for manova or group distance.
-		#' @param trans_otu default FALSE; whether species abundance will be square transformed, used for PCA.
-		#' @param ncomp default 3; the returned dimensions.
-		#' @param scale_species default FALSE; whether species loading in PCA will be scaled.
-		#' @return res_ordination stored in the object.
+		#' @param measure default NULL; bray, jaccard, wei_unifrac or unwei_unifrac, or other name of matrix you add; 
+		#' 	 beta diversity index used for ordination, manova or group distance.
+		#' @param group default NULL; sample group used for manova, betadisper or group distance.
+		#' @return parameters stored in the object.
 		#' @examples
 		#' data(dataset)
-		#' t1 <- trans_beta$new(dataset = dataset, ordination = "PCoA", measure = "bray", group = "Group")
-		initialize = function(dataset = NULL, ordination = NULL, measure = NULL, group = NULL, trans_otu = FALSE, ncomp = 3,
-			scale_species = FALSE
+		#' t1 <- trans_beta$new(dataset = dataset, measure = "bray", group = "Group")
+		initialize = function(
+			dataset = NULL, 
+			measure = NULL, 
+			group = NULL
 			) {
+			if(is.null(dataset)){
+				stop("dataset is necessary !")
+			}
+			if(!is.null(measure)){
+				if(!measure %in% c(names(dataset$beta_diversity), 1:length(dataset$beta_diversity))){
+					stop("Input measure should be one of beta_diversity distance in dataset !")
+				}else{
+					self$use_matrix <- dataset$beta_diversity[[measure]]
+				}
+			}
+			message("Please also cite the original paper: An et al. (2019). Soil bacterial community structure in Chinese wetlands. Geoderma, 337, 290-299.")
 			self$sample_table <- dataset$sample_table
 			self$measure <- measure
 			self$group <- group
-			if(!is.null(self$measure)){
-				if(!self$measure %in% c(names(dataset$beta_diversity), 1:length(dataset$beta_diversity))){
-					stop("Input measure should be one of beta_diversity distance in dataset")
+			use_dataset <- clone(dataset)
+			use_dataset$phylo_tree <- NULL
+			use_dataset$rep_fasta <- NULL
+			use_dataset$taxa_abund <- NULL
+			use_dataset$alpha_diversity <- NULL
+			self$dataset <- use_dataset
+		},
+		#' @description
+		#' Ordination based on An et al. (2019) <doi:10.1016/j.geoderma.2018.09.035>.
+		#'
+		#' @param ordination default "PCoA"; "PCA", "PCoA" or "NMDS".
+		#' @param ncomp default 3; the returned dimensions.
+		#' @param trans_otu default FALSE; whether species abundance will be square transformed, used for PCA.
+		#' @param scale_species default FALSE; whether species loading in PCA will be scaled.
+		#' @return res_ordination stored in the object.
+		#' @examples
+		#' t1$cal_ordination(ordination = "PCoA")		
+		cal_ordination = function(
+			ordination = "PCoA",
+			ncomp = 3,
+			trans_otu = FALSE, 
+			scale_species = FALSE		
+			){
+			if(is.null(ordination)){
+				stop("Input ordination should not be NULL !")
+			}
+			if(!ordination %in% c("PCA", "PCoA", "NMDS")){
+				stop("Input ordination should be one of 'PCA', 'PCoA' and 'NMDS' !")
+			}
+			dataset <- self$dataset
+			if(ordination == "PCA"){
+				plot.x <- "PC1"
+				plot.y <- "PC2"
+				if(trans_otu == T){
+					abund1 <- sqrt(dataset$otu_table)
 				}else{
-					self$use_matrix <- dataset$beta_diversity[[self$measure]]
+					abund1 <- dataset$otu_table
 				}
-			}
-			if(!is.null(ordination)){
-				if(!ordination %in% c("PCA", "PCoA", "NMDS")){
-					stop("Input ordination should be one of 'PCA', 'PCoA' and 'NMDS'")
-				}
-				if(ordination == "PCA"){
-					plot.x <- "PC1"
-					plot.y <- "PC2"
-					if(trans_otu == T){
-						abund1 <- sqrt(dataset$otu_table)
-					}else{
-						abund1 <- dataset$otu_table
-					}
-					model <- rda(t(abund1))
-					expla <- round(model$CA$eig/model$CA$tot.chi*100,1)
-					scores <- scores(model, choices = 1:ncomp)$sites
-					combined <- cbind.data.frame(scores, dataset$sample_table)
+				model <- rda(t(abund1))
+				expla <- round(model$CA$eig/model$CA$tot.chi*100,1)
+				scores <- scores(model, choices = 1:ncomp)$sites
+				combined <- cbind.data.frame(scores, dataset$sample_table)
 
+				if(is.null(dataset$tax_table)){
+					loading <- scores(model, choices = 1:ncomp)$species
+				}else{
 					loading <- cbind.data.frame(scores(model, choices = 1:ncomp)$species, dataset$tax_table)
-					loading <- cbind.data.frame(loading, rownames(loading))
+				}
+				loading <- cbind.data.frame(loading, rownames(loading))
 
-					if(scale_species == T){
-						maxx <- max(abs(scores[,plot.x]))/max(abs(loading[,plot.x]))
-						loading[, plot.x] <- loading[, plot.x] * maxx * 0.8
-						maxy <- max(abs(scores[,plot.y]))/max(abs(loading[,plot.y]))
-						loading[, plot.y] <- loading[, plot.y] * maxy * 0.8
-					}
+				if(scale_species == T){
+					maxx <- max(abs(scores[,plot.x]))/max(abs(loading[,plot.x]))
+					loading[, plot.x] <- loading[, plot.x] * maxx * 0.8
+					maxy <- max(abs(scores[,plot.y]))/max(abs(loading[,plot.y]))
+					loading[, plot.y] <- loading[, plot.y] * maxy * 0.8
+				}
 
-					species <- cbind(loading, loading[,plot.x]^2 + loading[,plot.y]^2)
-					colnames(species)[ncol(species)] <- "dist"
-					species <- species[with(species, order(-dist)), ]
-					outlist <- list(model = model, scores = combined, loading = species, eig = expla)
-				}
-				if(ordination == "PCoA"){
-					model <- ape::pcoa(as.dist(self$use_matrix))
-					combined <- cbind.data.frame(model$vectors[,1:ncomp], dataset$sample_table)
-					colnames(combined)[1:ncomp] <- c("PCo1", "PCo2","PCo3")[1:ncomp]
-					expla <- round(model$values[,1]/sum(model$values[,1])*100,1)
-					expla <- expla[1:ncomp]
-					names(expla) <- c("PCo1", "PCo2","PCo3")[1:ncomp]
-					outlist <- list(model = model, scores = combined, eig = expla)
-				}
-				if(ordination == "NMDS"){
-					model <- vegan::metaMDS(as.dist(self$use_matrix))
-					combined <- cbind.data.frame(model$points, dataset$sample_table)
-					outlist <- list(model = model, scores = combined)
-				}
-				self$res_ordination <- outlist
-				message('The ordination result is stored in object$res_ordination !')
+				species <- cbind(loading, loading[,plot.x]^2 + loading[,plot.y]^2)
+				colnames(species)[ncol(species)] <- "dist"
+				species <- species[with(species, order(-dist)), ]
+				outlist <- list(model = model, scores = combined, loading = species, eig = expla)
 			}
+			if(ordination %in% c("PCoA", "NMDS")){
+				if(is.null(self$use_matrix)){
+					stop("Please recreate the object and set the parameter measure !")
+				}
+			}
+			if(ordination == "PCoA"){
+				model <- ape::pcoa(as.dist(self$use_matrix))
+				combined <- cbind.data.frame(model$vectors[,1:ncomp], dataset$sample_table)
+				pco_names <- paste0("PCo", 1:10)
+				colnames(combined)[1:ncomp] <- pco_names[1:ncomp]
+				expla <- round(model$values[,1]/sum(model$values[,1])*100, 1)
+				expla <- expla[1:ncomp]
+				names(expla) <- pco_names[1:ncomp]
+				outlist <- list(model = model, scores = combined, eig = expla)
+			}
+			if(ordination == "NMDS"){
+				model <- vegan::metaMDS(as.dist(self$use_matrix))
+				combined <- cbind.data.frame(model$points, dataset$sample_table)
+				outlist <- list(model = model, scores = combined)
+			}
+			self$res_ordination <- outlist
+			message('The ordination result is stored in object$res_ordination ...')
 			self$ordination <- ordination
 		},
 		#' @description
@@ -124,18 +162,25 @@ trans_beta <- R6Class(classname = "trans_beta",
 			ellipse_type = "t"
 			){
 			ordination <- self$ordination
+			if(is.null(ordination)){
+				stop("Please first run cal_ordination function !")
+			}
+			
 			combined <- self$res_ordination$scores
 			eig <- self$res_ordination$eig
 			model <- self$res_ordination$model
 			plot_x <- colnames(self$res_ordination$scores)[1]
 			plot_y <- colnames(self$res_ordination$scores)[2]
+			
 			if (!is.null(plot_group_order)) {
 				combined[, plot_color] %<>% factor(., levels = plot_group_order)
 			}
+			
 			p <- ggplot(combined, aes_string(x = plot_x, y = plot_y, color = plot_color, shape = plot_shape))
 			p <- p + geom_point(alpha = plot_point_alpha, size = plot_point_size)
 			if(ordination %in% c("PCA", "PCoA")){
-				p <- p + xlab(paste(plot_x, " [", eig[plot_x],"%]", sep = "")) + ylab(paste(plot_y, " [", eig[plot_y],"%]", sep = ""))
+				p <- p + xlab(paste(plot_x, " [", eig[plot_x],"%]", sep = "")) + 
+					ylab(paste(plot_y, " [", eig[plot_y],"%]", sep = ""))
 			}
 			if(ordination == "NMDS"){
 				p <- p + annotate("text",x= max(combined[,1]),y=max(combined[,2]) + 0.05,label = round(model$stress, 2), parse=TRUE)
@@ -149,16 +194,27 @@ trans_beta <- R6Class(classname = "trans_beta",
 					dplyr::summarise(cx = mean(x), cy = mean(y)) %>%
 					as.data.frame()
 				os2 <- merge(combined, os, by.x= plot_group, by.y = "group")
-				p <- p + geom_segment(data=os2, aes_string(x = plot_x, xend = "cx", y = plot_y, yend = "cy", color = plot_group), alpha = segment_alpha, 
-					size = 0.5, linetype = centroid_linetype)
+				p <- p + geom_segment(
+					data=os2, 
+					aes_string(x = plot_x, xend = "cx", y = plot_y, yend = "cy", color = plot_group),
+					alpha = segment_alpha, 
+					size = 0.5, 
+					linetype = centroid_linetype
+				)
 			}
 			if (plot_group_ellipse == T) {
 				if(is.null(plot_group)){
 					plot_group <- plot_color
 				}
-				mapping = aes_string(x = plot_x, y = plot_y, group = plot_group, fill = plot_group)
-				p <- p + ggplot2::stat_ellipse(mapping = mapping, data = combined, level = ellipse_level, type = ellipse_type, 
-					alpha = ellipse_alpha, geom = "polygon") + 
+				mapping <- aes_string(x = plot_x, y = plot_y, group = plot_group, fill = plot_group)
+				p <- p + ggplot2::stat_ellipse(
+					mapping = mapping, 
+					data = combined, 
+					level = ellipse_level, 
+					type = ellipse_type, 
+					alpha = ellipse_alpha, 
+					geom = "polygon"
+					) + 
 					scale_fill_manual(values = color_values)
 			}
 			if(!is.null(plot_color)){
@@ -182,7 +238,15 @@ trans_beta <- R6Class(classname = "trans_beta",
 		#' @return res_manova stored in object.
 		#' @examples
 		#' t1$cal_manova(cal_manova_all = TRUE)
-		cal_manova = function(cal_manova_all = FALSE, cal_manova_paired = FALSE, cal_manova_set = NULL, permutations = 999){
+		cal_manova = function(
+			cal_manova_all = FALSE, 
+			cal_manova_paired = FALSE, 
+			cal_manova_set = NULL, 
+			permutations = 999
+			){
+			if(is.null(self$use_matrix)){
+				stop("Please recreate the object and set the parameter measure !")
+			}
 			use_matrix <- self$use_matrix
 			if(!is.null(cal_manova_set)){
 				self$res_manova <- adonis(reformulate(cal_manova_set, substitute(as.dist(use_matrix))), data = self$sample_table, permutations = permutations)
@@ -191,10 +255,32 @@ trans_beta <- R6Class(classname = "trans_beta",
 				self$res_manova <- adonis(reformulate(self$group, substitute(as.dist(use_matrix))), data = self$sample_table, permutations = permutations)
 			}
 			if(cal_manova_paired == T){
-				self$res_manova <- private$paired_group_manova(sample_info_use = self$sample_table, use_matrix = use_matrix, 
-					group = self$group, measure = self$measure, permutations = permutations)
+				self$res_manova <- private$paired_group_manova(
+					sample_info_use = self$sample_table, 
+					use_matrix = use_matrix, 
+					group = self$group, 
+					measure = self$measure, 
+					permutations = permutations
+				)
 			}
-			message('The result is stored in object$res_manova !')
+			message('The result is stored in object$res_manova ...')
+		},
+		#' @description
+		#' A wrapper for betadisper function in vegan package for multivariate homogeneity test of groups dispersions.
+		#'
+		#' @param ... parameters passed to \code{\link{betadisper}} function.
+		#' @return res_betadisper stored in object.
+		#' @examples
+		#' t1$cal_betadisper()
+		cal_betadisper = function(...){
+			if(is.null(self$use_matrix)){
+				stop("Please recreate the object and set the parameter measure !")
+			}
+			use_matrix <- self$use_matrix
+			res1 <- betadisper(as.dist(use_matrix), self$sample_table[, self$group], ...)
+			res2 <- permutest(res1, pairwise = TRUE)
+			self$res_betadisper <- res2
+			message('The result is stored in object$res_betadisper ...')
 		},
 		#' @description
 		#' Transform sample distances within groups or between groups.
@@ -211,7 +297,7 @@ trans_beta <- R6Class(classname = "trans_beta",
 			}else{
 				self$res_group_distance <- private$between_group_distance(distance = self$use_matrix, sampleinfo=self$sample_table, type = self$group)
 			}
-			message('The result is stored in object$res_group_distance !')
+			message('The result is stored in object$res_group_distance ...')
 		},
 		#' @description
 		#' Plotting the distance between samples within or between groups.
@@ -227,7 +313,8 @@ trans_beta <- R6Class(classname = "trans_beta",
 		#' \donttest{
 		#' t1$plot_group_distance(distance_pair_stat = TRUE)
 		#' }
-		plot_group_distance = function(plot_group_order = NULL,
+		plot_group_distance = function(
+			plot_group_order = NULL,
 			color_values = RColorBrewer::brewer.pal(8, "Dark2"),
 			distance_pair_stat = FALSE,
 			pair_compare_filter = "",
@@ -237,7 +324,11 @@ trans_beta <- R6Class(classname = "trans_beta",
 			group_distance <- self$res_group_distance
 			group <- self$group
 			if(self$measure %in% c("wei_unifrac", "unwei_unifrac", "bray", "jaccard")){
-				titlename <- switch(self$measure, wei_unifrac = "Weighted Unifrac", unwei_unifrac = "Unweighted Unifrac", bray = "Bray-Curtis", jaccard = "Jaccard")
+				titlename <- switch(self$measure, 
+					wei_unifrac = "Weighted Unifrac", 
+					unwei_unifrac = "Unweighted Unifrac", 
+					bray = "Bray-Curtis", 
+					jaccard = "Jaccard")
 				ylabname <- paste0(titlename, " distance")
 			}else{
 				ylabname <- self$measure
@@ -275,19 +366,26 @@ trans_beta <- R6Class(classname = "trans_beta",
 		#' Plotting clustering result. Require ggdendro package.
 		#'
 		#' @param use_colors colors for presentation.
-		#' @param measure default NULL; beta diversity index; suggest using the measure when creating object
+		#' @param measure default NULL; beta diversity index; If NULL, using the measure when creating object
 		#' @param group default NULL; if provided, use this group to assign color.
 		#' @param replace_name default NULL; if provided, use this as label.
 		#' @return ggplot.
 		#' @examples
 		#' t1$plot_clustering(group = "Group", replace_name = c("Saline", "Type"))
-		plot_clustering = function(use_colors = RColorBrewer::brewer.pal(8, "Dark2"), measure = NULL, group = NULL, replace_name = NULL){
+		plot_clustering = function(
+			use_colors = RColorBrewer::brewer.pal(8, "Dark2"), 
+			measure = NULL, 
+			group = NULL, 
+			replace_name = NULL
+			){
+			dataset <- self$dataset
 			if(is.null(measure)){
-				if(is.null(self$measure)){
+				if(is.null(self$use_matrix)){
 					measure_matrix <- dataset$beta_diversity[[1]]
+					measure <- names(dataset$beta_diversity)[1]
 				}else{
+					measure_matrix <- self$use_matrix
 					measure <- self$measure
-					measure_matrix <- dataset$beta_diversity[[measure]]
 				}
 			}else{
 				measure_matrix <- dataset$beta_diversity[[measure]]
@@ -297,7 +395,8 @@ trans_beta <- R6Class(classname = "trans_beta",
 			titlename <- switch(measure, wei_unifrac = "Weighted Unifrac", unwei_unifrac = "Unweighted Unifrac", bray = "Bray-Curtis", jaccard = "Jaccard")
 			ylabname <- paste0("Distance (", titlename, ")")
 
-			g1 <- ggplot(data = ggdendro::segment(hc_d_measure)) + geom_segment(aes(x=x, y=y, xend=xend, yend=yend), color = "grey30")
+			g1 <- ggplot(data = ggdendro::segment(hc_d_measure)) + 
+				geom_segment(aes(x=x, y=y, xend=xend, yend=yend), color = "grey30")
 			if(!is.null(group) | !is.null(replace_name)){
 				data2 <- suppressWarnings(dplyr::left_join(hc_d_measure$label, rownames_to_column(self$sample_table), by = c("label" = "rowname")))
 				if(length(replace_name) > 1){
@@ -403,5 +502,3 @@ trans_beta <- R6Class(classname = "trans_beta",
 	lock_class = FALSE,
 	lock_objects = FALSE
 )
-
-
