@@ -4,11 +4,16 @@
 #' @description
 #' This class is a wrapper for a series of network analysis related methods, 
 #' including the correlation based <doi:10.1186/1471-2105-13-113>, SpiecEasi <doi:10.1371/journal.pcbi.1004226>,
-#' and Probabilistic Graphical Models based <doi:10.1016/j.cels.2019.08.002> network construction approaches, network and node attributes analysis and other network operations.
+#' and Probabilistic Graphical Models based <doi:10.1016/j.cels.2019.08.002> network construction approaches, network and node attributes analysis
+#' eigengene analysis, network subsetting and other network operations.
 #'
 #' @export
 trans_network <- R6Class(classname = "trans_network",
 	public = list(
+		#' @description
+		#' This function is used to create the trans_network object, store the important intermediate data 
+		#'   and calculate correlations if cal_cor parameter is selected.
+		#' 
 		#' @param dataset the object of \code{\link{microtable}} Class.
 		#' @param cor_method default "pearson"; "pearson", "spearman" or "kendall"; correlation algorithm, only use for correlation based network.
 		#' @param cal_cor default "base"; "base", "WGCNA", "SparCC" or NA; correlation method; NA represent do not calculate correlations, used for non-correlation based network. 
@@ -23,8 +28,11 @@ trans_network <- R6Class(classname = "trans_network",
 		#' \donttest{
 		#' data(dataset)
 		#' # correlation network
-		#' t1 <- trans_network$new(dataset = dataset, cal_cor = "base", 
-		#'   taxa_level = "OTU", filter_thres = 0.001)
+		#' t1 <- trans_network$new(
+		#' 		dataset = dataset, 
+		#' 		cal_cor = "base", 
+		#' 		taxa_level = "OTU", 
+		#' 		filter_thres = 0.001)
 		#' }
 		initialize = function(
 			dataset = NULL,
@@ -57,8 +65,9 @@ trans_network <- R6Class(classname = "trans_network",
 			if(taxa_level != "OTU"){
 				dataset1 <- dataset1$merge_taxa(taxa = taxa_level)
 			}
-			# transform data
+			# transform each object
 			sampleinfo <- dataset1$sample_table
+			# store taxonomic table for the following analysis
 			self$use_tax <- dataset1$tax_table
 			use_abund <- dataset1$otu_table
 			use_abund <- use_abund[apply(use_abund, 1, sum)/sum(use_abund) > filter_thres, ]
@@ -82,16 +91,27 @@ trans_network <- R6Class(classname = "trans_network",
 					com_res <- t(combn(use_names, 2))
 					res <- cbind.data.frame(com_res, cor_result$cors, cor_result$pvals, stringsAsFactors = FALSE)
 					colnames(res) <- c("t1", "t2", "cor", "p")
-					res <- rbind.data.frame(res, data.frame(t1 = res$t2, t2 = res$t1, cor = res$cor, p = res$p), 
-						data.frame(t1 = use_names, t2 = use_names, cor = rep(1, length(use_names)), p = rep(0, length(use_names))))
+					res <- rbind.data.frame(
+						res, 
+						data.frame(t1 = res$t2, t2 = res$t1, cor = res$cor, p = res$p), 
+						data.frame(t1 = use_names, t2 = use_names, cor = rep(1, length(use_names)), p = rep(0, length(use_names)))
+						)
 					res$cor %<>% as.numeric
 					res$p %<>% as.numeric
-					res_cor <- reshape2::dcast(res, t1~t2, value.var = "cor") %>% `row.names<-`(.[,1]) %>% .[, -1] %>% .[use_names, use_names] %>% as.matrix
-					res_p <- reshape2::dcast(res, t1~t2, value.var = "p") %>% `row.names<-`(.[,1]) %>% .[, -1] %>% .[use_names, use_names] %>% as.matrix
+					res_cor <- reshape2::dcast(res, t1~t2, value.var = "cor") %>% 
+						`row.names<-`(.[,1]) %>% 
+						.[, -1] %>% 
+						.[use_names, use_names] %>% 
+						as.matrix
+					res_p <- reshape2::dcast(res, t1~t2, value.var = "p") %>% 
+						`row.names<-`(.[,1]) %>% 
+						.[, -1] %>% 
+						.[use_names, use_names] %>% 
+						as.matrix
 					cor_result <- list(cor = res_cor, p = res_p)
 				}
 				self$res_cor_p <- cor_result
-				message('The correlation result list is stored in object$res_cor_p !')
+				message('The correlation result list is stored in object$res_cor_p ...')
 			}else{
 				self$res_cor_p <- NULL
 			}
@@ -102,9 +122,9 @@ trans_network <- R6Class(classname = "trans_network",
 		},
 		#' @description
 		#' Calculate network either based on the correlation method or based on SpiecEasi or based on the Probabilistic Graphical Models (PGM) in julia FlashWeave; 
-		#' see Deng et al. (2012) <doi:10.1186/1471-2105-13-113> for correlation based method; 
-		#' see Kurtz et al. (2015) <doi:doi:10.1371/journal.pcbi.1004226> for SpiecEasi method; 
-		#' see Tackmann et al. (2019) <doi:10.1016/j.cels.2019.08.002> for PGM based method.
+		#' See Deng et al. (2012) <doi:10.1186/1471-2105-13-113> for correlation based method, 
+		#' Kurtz et al. (2015) <doi:doi:10.1371/journal.pcbi.1004226> for SpiecEasi method, 
+		#' Tackmann et al. (2019) <doi:10.1016/j.cels.2019.08.002> for PGM based method.
 		#'
 		#' @param network_method default "COR"; "COR", "SpiecEasi" or "PGM"; COR: correlation based method; PGM: Probabilistic Graphical Models based method.
 		#' @param p_thres default .01; the p value threshold.
@@ -117,9 +137,9 @@ trans_network <- R6Class(classname = "trans_network",
 		#' @param PGM_sensitive default "true"; whether use sensitive type in the PGM model.
 		#' @param PGM_heterogeneous default "true"; whether use heterogeneous type in the PGM model.
 		#' @param SpiecEasi_method default "mb"; either 'glasso' or 'mb';see spiec.easi in package SpiecEasi and https://github.com/zdk123/SpiecEasi.
-		#' @param add_taxa_name default "Phylum"; add taxonomic rank name to the result.
+		#' @param add_taxa_name default "Phylum"; NULL or a taxonomic rank name; used to add taxonomic rank name to network.
 		#' @param usename_rawtaxa_when_taxalevel_notOTU default FALSE; whether replace the name of nodes using the taxonomic information.
-		#' @param ... paremeters pass to spiec.easi in package SpiecEasi.
+		#' @param ... paremeters pass to spiec.easi in package SpiecEasi for network_method = "SpiecEasi".
 		#' @return res_network in object.
 		#' @examples
 		#' \donttest{
@@ -150,16 +170,16 @@ trans_network <- R6Class(classname = "trans_network",
 			if(!grepl("COR|PGM|SpiecEasi", network_method, ignore.case = TRUE)){
 				stop("Unknown network method in network_method parameter!")
 			}
+			
+			message("---------------- ", Sys.time()," : Start ----------------")
 			if(grepl("COR", network_method, ignore.case = TRUE)){
 				cortable <- self$res_cor_p$cor
 				adp <- apply(self$res_cor_p$p, 2, p.adjust, method = COR_p_adjust)
 				if(COR_optimization == T) {
 					#find out threshold of correlation 
-					print(paste(Sys.time(),"calculate COR threshold: start"))
 					tc1 <- private$rmt(cortable)
-					print(paste(Sys.time(),"calculate COR threshold: finished"))
 					tc1 <- ifelse(tc1 > COR_low_threshold, tc1, COR_low_threshold)
-					message("The optimized COR threshold: ", tc1, "\n")
+					message("The optimized COR threshold: ", tc1, "...\n")
 				}
 				else {
 					tc1 <- COR_cut
@@ -222,36 +242,50 @@ trans_network <- R6Class(classname = "trans_network",
 				close(openfile)
 				system("julia calculate_network.jl")
 				setwd('..')
+				message("The temporary files are in ", tem_dir)
 				network <- read_graph(paste0(tem_dir, "/network_PGM.gml"), format = "gml")
 				network <- set_vertex_attr(network, "name", value = V(network)$label)
 				E(network)$label <- unlist(lapply(E(network)$weight, function(x) ifelse(x > 0, "+", "-")))
 				E(network)$weight <- abs(E(network)$weight)
 			}
+			message("---------------- ", Sys.time()," : Finish ----------------")
+			
 			nodes_raw <- data.frame(cbind(V(network), V(network)$name))
+			# delete uncultured taxa when the taxa level is not OTU
 			if(taxa_level != "OTU"){
-				delete_nodes <- taxa_table %>% .[grepl("__$|uncultured", .[, taxa_level]), ] %>% rownames %>% .[. %in% rownames(nodes_raw)]
+				delete_nodes <- taxa_table %>% 
+					.[grepl("__$|uncultured", .[, taxa_level]), ] %>% 
+					rownames %>% 
+					.[. %in% rownames(nodes_raw)]
 				network %<>% delete_vertices(delete_nodes)
 				nodes_raw <- data.frame(cbind(V(network), V(network)$name))
 			}
 			edges <- t(sapply(1:ecount(network), function(x) ends(network, x)))
-			delete_nodes <- rownames(nodes_raw) %>% .[! . %in% as.character(c(edges[,1], edges[,2]))]
+			delete_nodes <- rownames(nodes_raw) %>% 
+				.[! . %in% as.character(c(edges[,1], edges[,2]))]
 			network %<>% delete_vertices(delete_nodes)
 			V(network)$taxa <- V(network)$name
 			if(!is.null(add_taxa_name)){
-				network <- set_vertex_attr(network, add_taxa_name, value = V(network)$name %>% taxa_table[., add_taxa_name] %>% gsub("^.__", "", .))
+				network <- set_vertex_attr(network, add_taxa_name, value = V(network)$name %>% 
+					taxa_table[., add_taxa_name] %>% 
+					gsub("^.__", "", .))
 			}
 			if(taxa_level != "OTU"){
 				if(usename_rawtaxa_when_taxalevel_notOTU == T){
-					network <- set_vertex_attr(network, taxa_level, value = V(network)$name %>% taxa_table[., taxa_level] %>% gsub("^.__", "", .))
+					network <- set_vertex_attr(network, taxa_level, value = V(network)$name %>% 
+						taxa_table[., taxa_level] %>% 
+						gsub("^.__", "", .))
 				}else{
-					network <- set_vertex_attr(network, "name", value = V(network)$name %>% taxa_table[., taxa_level] %>% gsub("^.__", "", .))
+					network <- set_vertex_attr(network, "name", value = V(network)$name %>% 
+						taxa_table[., taxa_level] %>% 
+						gsub("^.__", "", .))
 				}
 			}
 			self$res_network <- network
-			message('The result network is stored in object$res_network !')
+			message('The result network is stored in object$res_network ...')
 		},
 		#' @description
-		#' Calculate and add network modules.
+		#' Add network modules to the network.
 		#'
 		#' @param module_name_prefix default "M"; the prefix of module names; module names are made of the module_name_prefix and numbers;
 		#'   numbers are assigned according to the sorting result of node numbers in modules with decreasing trend.
@@ -268,9 +302,9 @@ trans_network <- R6Class(classname = "trans_network",
 			for(i in seq_along(mod2)){
 				mod1[mod1 == names(mod2)[i]] <- paste0(module_name_prefix, i)
 			}
-			message('Totally, ', length(mod2), ' modules are idenfified')
+			message('Totally, ', length(mod2), ' modules are idenfified ...')
 			network <- set_vertex_attr(network, "module", value = mod1)
-			message('Modules are assigned in network with attribute name -- module')
+			message('Modules are assigned in network with attribute name -- module ...')
 			self$res_network <- network
 		},
 		#' @description
@@ -294,7 +328,7 @@ trans_network <- R6Class(classname = "trans_network",
 		#' }
 		cal_network_attr = function(){
 			self$res_network_attr <- private$network_attribute(self$res_network)
-			message('Result is stored in object$res_network_attr !')
+			message('Result is stored in object$res_network_attr ...')
 		},
 		#' @description
 		#' Calculate node properties.
@@ -308,6 +342,7 @@ trans_network <- R6Class(classname = "trans_network",
 			network <- self$res_network
 			node_type <- private$module_roles(network)
 			use_abund <- self$use_abund
+			# create a replace_table to match the taxa name and marker name when taxa_level is not "OTU"
 			if(self$taxa_level != "OTU"){
 				replace_table <- data.frame(V(network)$name, V(network)$taxa, stringsAsFactors = FALSE) %>% `row.names<-`(.[,1])
 				node_type <- cbind.data.frame(node_type, 
@@ -317,9 +352,17 @@ trans_network <- R6Class(classname = "trans_network",
 			}
 			node_type$degree <- degree(network)[rownames(node_type)]
 			node_type$betweenness <- betweenness(network)[rownames(node_type)]
-			node_type$Abundance <- apply(use_abund, 2, function(x) sum(x) * 100/sum(use_abund))[rownames(node_type)]
+			# Add abundance info
+			sum_abund <- apply(use_abund, 2, function(x) sum(x) * 100/sum(use_abund))
+			# Same with the above operation to make the names corresponded
+			if(self$taxa_level != "OTU"){
+				node_type$Abundance <- sum_abund[replace_table[rownames(node_type), 2]]
+			}else{
+				node_type$Abundance <- sum_abund[rownames(node_type)]
+			}
+			
 			self$res_node_type <- node_type
-			message('Result is stored in object$res_node_type !')
+			message('Result is stored in object$res_node_type ...')
 		},
 		#' @description
 		#' Calculate eigengenes of modules, i.e. the first principal component based on PCA analysis, and the percentage of variance.
@@ -340,6 +383,11 @@ trans_network <- R6Class(classname = "trans_network",
 				if(length(tax_names) < 3){
 					next
 				}
+				if(self$taxa_level != "OTU"){
+					network <- self$res_network
+					replace_table <- data.frame(V(network)$name, V(network)$taxa, stringsAsFactors = FALSE) %>% `row.names<-`(.[,1])
+					tax_names <- replace_table[tax_names, 2]
+				}
 				sel_abund <- use_abund[, tax_names]
 				pca_model <- rda(sel_abund)
 				sel_scores <- scores(pca_model, choices = 1)$sites %>% as.data.frame
@@ -352,12 +400,12 @@ trans_network <- R6Class(classname = "trans_network",
 			res_eigen <- do.call(cbind, res_eigen)
 			self$res_eigen <- res_eigen
 			self$res_eigen_expla <- res_eigen_expla
-			message('Result is stored in object$res_eigen and object$res_eigen_expla !')
+			message('Result is stored in object$res_eigen and object$res_eigen_expla ...')
 		},
 		#' @description
-		#' Plot the classification and importance of nodes.
+		#' Plot the classification and importance of nodes, see object$res_node_type for the variable names used in the parameters.
 		#'
-		#' @param use_type default 1; 1 or 2; 1 represent the traditional taxa roles plot; 2 represent the plot with taxa names as x axis.
+		#' @param use_type default 1; 1 or 2; 1 represents taxa roles area plot; 2 represents the layered plot with taxa as x axis.
 		#' @param roles_colors default NULL; for use_type 1; colors for each group.
 		#' @param plot_module default FALSE; for use_type 1; whether plot the modules information.
 		#' @param use_level default "Phylum"; for use_type 2; used taxonomic level in x axis.
@@ -365,7 +413,7 @@ trans_network <- R6Class(classname = "trans_network",
 		#' @param show_number default 1:10; for use_type 2; showed number in x axis, sorting according to the nodes number.
 		#' @param plot_color default "Phylum"; for use_type 2; used variable for color.
 		#' @param plot_shape default "taxa_roles"; for use_type 2; used variable for shape.
-		#' @param plot_size default NULL; for use_type 2; used variable for shape.
+		#' @param plot_size default "Abundance"; for use_type 2; used for point size; a fixed number (e.g. 5) is also available.
 		#' @param color_values default RColorBrewer::brewer.pal(12, "Paired"); for use_type 2; color vector
 		#' @param shape_values default c(16, 17, 7, 8, 15, 18, 11, 10, 12, 13, 9, 3, 4, 0, 1, 2, 14); for use_type 2; shape vector, see ggplot2 tutorial for the shape meaning.
 		#' @return ggplot.
@@ -382,7 +430,7 @@ trans_network <- R6Class(classname = "trans_network",
 			show_number = 1:10,
 			plot_color = "Phylum",
 			plot_shape = "taxa_roles",
-			plot_size = NULL,
+			plot_size = "Abundance",
 			color_values = RColorBrewer::brewer.pal(12, "Paired"),
 			shape_values = c(16, 17, 7, 8, 15, 18, 11, 10, 12, 13, 9, 3, 4, 0, 1, 2, 14)
 			){
@@ -391,71 +439,19 @@ trans_network <- R6Class(classname = "trans_network",
 			}
 			if(use_type == 2){
 				res <- private$plot_roles_2(node_roles = self$res_node_type, 
-					plot_color = plot_color, plot_shape = plot_shape,
-					use_level = use_level, show_value = show_value, show_number = show_number, 
-					color_values = color_values, shape_values = shape_values
+					plot_color = plot_color,
+					plot_shape = plot_shape,
+					use_level = use_level, 
+					show_value = show_value, 
+					show_number = show_number,
+					plot_size = plot_size,
+					color_values = color_values, 
+					shape_values = shape_values
 				)
 			}
 			res
 		},
 		#' @description
-		#' This function is used to sum the links number from one taxa to another or in the same taxa, for example, at Phylum level.
-		#' This is very useful to fast see how many nodes are connected between different taxa or within the taxa.
-		#'
-		#' @param taxa_level default "Phylum"; taxonomic rank.
-		#' @return res_sum_links_pos and res_sum_links_neg in object.
-		#' @examples
-		#' \donttest{
-		#' t1$cal_sum_links(taxa_level = "Phylum")
-		#' }
-		cal_sum_links = function(taxa_level = "Phylum"){
-			taxa_table <- self$use_tax
-			network <- self$res_network
-			link_table <- data.frame(t(sapply(1:ecount(network), function(x) ends(network, x))), label = E(network)$label, stringsAsFactors = FALSE)
-			if("+" %in% link_table[, 3]){
-				link_table_1 <- link_table[link_table[, 3] %in% "+", ]
-				self$res_sum_links_pos <- private$sum_link(taxa_table = taxa_table, link_table = link_table_1, taxa_level = taxa_level)
-			}
-			if("-" %in% link_table[, 3]){
-				link_table_1 <- link_table[link_table[, 3] %in% "-", ]
-				self$res_sum_links_neg <- private$sum_link(taxa_table = taxa_table, link_table = link_table_1, taxa_level = taxa_level)
-			}
-			message('The result is stored in object$res_sum_links_pos and/or object$res_sum_links_neg !')
-		},
-		#' @description
-		#' Plot the summed linkages among taxa using chorddiag package <https://github.com/mattflor/chorddiag>.
-		#'
-		#' @param plot_pos default TRUE; plot the summed positive or negative linkages.
-		#' @param plot_num default NULL; number of taxa presented in the plot.
-		#' @param color_values default NULL; If not provided, use default.
-		#' @return chorddiag plot
-		plot_sum_links = function(plot_pos = TRUE, plot_num = NULL, color_values = NULL){
-			if(plot_pos == T){
-				if(is.null(self$res_sum_links_pos)){
-					stop("No res_sum_links_pos found!\n")
-				}else{
-					use_data <- self$res_sum_links_pos
-				}
-			}else{
-				if(is.null(self$res_sum_links_neg)){
-					stop("No res_sum_links_neg found!\n")
-				}else{
-					use_data <- self$res_sum_links_neg
-				}
-			}
-			if(!is.null(plot_num)){
-				use_data %<>% .[1:plot_num, 1:plot_num]
-			}
-			if(is.null(color_values)){
-				if(nrow(use_data) <= 14){
-					groupColors <- c(RColorBrewer::brewer.pal(12, "Paired"), "#FDE0EF", "#C51B7D")
-				}else{
-					groupColors <- unname(randomcoloR::distinctColorPalette(nrow(use_data)))
-				}
-			}
-			chorddiag::chorddiag(use_data, groupColors = groupColors)
-		},
-		#' @description
 		#' Subset of the network.
 		#'
 		#' @param node default NULL; provide the node names that you want to use in the sub-network.
@@ -527,9 +523,13 @@ trans_network <- R6Class(classname = "trans_network",
 				cor_res <- suppressWarnings(cor.test(inputtable[, com_res[1, x]], inputtable[, com_res[2, x]], method = cor_method));
 				c(t1 = com_res[1, x], t2 = com_res[2, x], cor = unname(cor_res$estimate), p = unname(cor_res$p.value))
 			})
-			res <- do.call(rbind, res) %>% as.data.frame(stringsAsFactors = FALSE)
-			res <- rbind.data.frame(res, data.frame(t1 = res$t2, t2 = res$t1, cor = res$cor, p = res$p), 
-				data.frame(t1 = use_names, t2 = use_names, cor = rep(1, N), p = rep(0, N)))
+			res <- do.call(rbind, res) %>% 
+				as.data.frame(stringsAsFactors = FALSE)
+			res <- rbind.data.frame(
+				res, 
+				data.frame(t1 = res$t2, t2 = res$t1, cor = res$cor, p = res$p), 
+				data.frame(t1 = use_names, t2 = use_names, cor = rep(1, N), p = rep(0, N))
+				)
 			res$cor %<>% as.numeric
 			res$p %<>% as.numeric
 			res_cor <- reshape2::dcast(res, t1~t2, value.var = "cor") %>% `row.names<-`(.[,1]) %>% .[, -1] %>% .[use_names, use_names] %>% as.matrix
@@ -538,32 +538,32 @@ trans_network <- R6Class(classname = "trans_network",
 			res
 		},
 		rmt = function(cormat,lcor=0.4, hcor=0.8){
-			nnsd <- function(sp = spp){
-				nns<-NULL
-				for(j in 2:length(sp)){
-					nn=abs(sp[j] - sp[j-1])
-					nns<-c(nns, nn)
-				}
-				return(nns)
-			}
-			s<-seq(0,3,0.1)
-			pois<-exp(-s)
-			geo<-0.5*pi*s*exp(-0.25*pi*s^2)
-			ps<-NULL  
-			for (i in seq(lcor,hcor,0.01)){
+			s <- seq(0, 3, 0.1)
+			pois <- exp(-s)
+			geo <- 0.5 * pi * s * exp(-0.25 * pi * s^2)
+			ps <- NULL  
+			for(i in seq(lcor, hcor, 0.01)){
 				cormat1 <- abs(cormat)
-				cormat1[cormat1<i] <- 0  
-				eigen = sort(eigen(cormat1)$value)
-				ssp <- smooth.spline(eigen, control.spar = list(low = 0,high = 3)) 
-				nnsd1<-density(nnsd(ssp$y))
-				nnsdpois<-density(nnsd(pois))
-				chival1<-sum((nnsd1$y - nnsdpois$y)^2/nnsdpois$y/512)
-				ps<-rbind(ps,chival1)
+				cormat1[cormat1 < i] <- 0  
+				eigen <- sort(eigen(cormat1)$value)
+				ssp <- smooth.spline(eigen, control.spar = list(low = 0, high = 3)) 
+				nnsd1 <- density(private$nnsd(ssp$y))
+				nnsdpois <- density(private$nnsd(pois))
+				chival1 <- sum((nnsd1$y - nnsdpois$y)^2/nnsdpois$y/512)
+				ps <- rbind(ps, chival1)
 				print(i*100)
 			}
-			ps <- cbind(ps,c(seq(lcor,hcor,0.01)))
-			tc <- ps[ps[,1]==min(ps[,1]),2]
-			return(tc)
+			ps <- cbind(ps, c(seq(lcor, hcor, 0.01)))
+			tc <- ps[ps[,1] == min(ps[,1]), 2]
+			tc
+		},
+		nnsd = function(sp){
+			nns <- NULL
+			for(j in 2:length(sp)){
+				nn <- abs(sp[j] - sp[j-1])
+				nns <- c(nns, nn)
+			}
+			nns
 		},
 		saveAsGEXF = function(network, filepath = "network.gexf"){
 			require(rgexf)
@@ -601,6 +601,7 @@ trans_network <- R6Class(classname = "trans_network",
 		},
 		# modified based on microbiomeSeq (http://www.github.com/umerijaz/microbiomeSeq) 
 		module_roles = function(comm_graph){
+			require(igraph)
 			td <- degree(comm_graph) %>% data.frame(taxa = names(.), total_links = ., stringsAsFactors = FALSE)
 			wmd <- private$within_module_degree(comm_graph)
 			z <- private$zscore(wmd)
@@ -608,26 +609,35 @@ trans_network <- R6Class(classname = "trans_network",
 			pc <- private$participation_coeffiecient(amd, td)
 			zp <- data.frame(z, pc)
 			nod_roles <- private$assign_module_roles(zp)
-			return(nod_roles)
+			nod_roles
 		},
 		#compute within-module degree for each of the features
 		within_module_degree = function(comm_graph){
 			mods <- get.vertex.attribute(comm_graph, "module")
+			if(is.null(mods)){
+				stop("No modules found! Please first calculate network modules using function cal_module!")
+			}
 			modvs <- data.frame("taxon"= V(comm_graph)$name, "mod"=mods, stringsAsFactors = FALSE)
 			sg1 <- decompose.graph(comm_graph, mode="strong")
 			res <- data.frame()
 			for(mod in unique(modvs$mod)){
-				mod_nodes <- subset(modvs$taxon,modvs$mod==mod)
-				neighverts <- unique(unlist(sapply(sg1,FUN=function(s){if(any(V(s)$name %in% mod_nodes)) V(s)$name else NULL})))
-				g3 <- induced.subgraph(graph=comm_graph,vids=neighverts)
+				mod_nodes <- subset(modvs$taxon, modvs$mod == mod)
+				neighverts <- unique(unlist(sapply(sg1, FUN = function(s){
+					if(any(V(s)$name %in% mod_nodes)){
+						V(s)$name
+						}else{
+						NULL
+						}
+					})))
+				g3 <- induced.subgraph(graph = comm_graph, vids = neighverts)
 				mod_degree <- degree(g3)
 				for(i in mod_nodes){
-					ki <- mod_degree[which(names(mod_degree)==i)]
-					tmp <- data.frame(module=mod, taxa=names(ki), mod_links=ki)
+					ki <- mod_degree[which(names(mod_degree) == i)]
+					tmp <- data.frame(module = mod, taxa = names(ki), mod_links = ki)
 					res <- rbind(res,tmp)
 				}
 			}
-			return(res)
+			res
 		},
 		#compute within-module degree z-score which
 		#measures how well-connected a node is to other nodes in the module.
@@ -641,7 +651,7 @@ trans_network <- R6Class(classname = "trans_network",
 				z[i] <- (mod.degree$mod_links[i] - mod_mean)/mod_sig
 			}
 			z <- data.frame(row.names=rownames(mod.degree), z, module=mod.degree$module)
-			return(z)
+			z
 		},
 		#calculate the degree (links) of each node to nodes in other modules.
 		among_module_connectivity = function(comm_graph){
@@ -652,8 +662,12 @@ trans_network <- R6Class(classname = "trans_network",
 							c(taxa = x, module = modvs[modvs$taxon==y, "mod"])
 						})
 					})
-			res <- do.call(cbind, res) %>% t %>% as.data.frame(stringsAsFactors = FALSE) %>% 
-				dplyr::group_by(taxa) %>% dplyr::count(module) %>% as.data.frame(stringsAsFactors = FALSE)
+			res <- do.call(cbind, res) %>% 
+				t %>% 
+				as.data.frame(stringsAsFactors = FALSE) %>% 
+				dplyr::group_by(taxa) %>% 
+				dplyr::count(module) %>% 
+				as.data.frame(stringsAsFactors = FALSE)
 			colnames(res)[colnames(res) == "n"] <- c("mod_links")
 			res
 		},
@@ -663,8 +677,8 @@ trans_network <- R6Class(classname = "trans_network",
 		participation_coeffiecient = function(mod.degree, total.degree){
 			p <- NULL
 			for(i in total.degree$taxa){
-				ki <- subset(total.degree$total_links, total.degree$taxa==i)
-				taxa.mod.degree <- subset(mod.degree$mod_links, mod.degree$taxa==i)
+				ki <- subset(total.degree$total_links, total.degree$taxa == i)
+				taxa.mod.degree <- subset(mod.degree$mod_links, mod.degree$taxa == i)
 				p[i] <- 1 - (sum((taxa.mod.degree)**2)/ki**2)
 			}
 			p <- as.data.frame(p)
@@ -718,7 +732,9 @@ trans_network <- R6Class(classname = "trans_network",
 			}else{
 				p <- p + geom_point(data=node_roles, aes(x=p, y=z)) + theme_bw()
 			}
-			p <- p + theme(strip.background = element_rect(fill = "white")) + xlab("Among-module connectivity") + ylab(" Within-module connectivity")
+			p <- p + theme(strip.background = element_rect(fill = "white")) + 
+				xlab("Among-module connectivity") + 
+				ylab(" Within-module connectivity")
 			p
 		},
 		# plot z and p according to the taxonomic levels
@@ -748,7 +764,10 @@ trans_network <- R6Class(classname = "trans_network",
 			# filter, subset and sort the shown taxa
 			use_data <- use_data[!grepl("__$", as.character(use_data[, use_level])), ]
 			use_data[, use_level] %<>% gsub(".__", "", .)
-			use_taxa <- table(as.character(use_data[, use_level])) %>% sort(decreasing = TRUE) %>% names %>% .[show_number]
+			use_taxa <- table(as.character(use_data[, use_level])) %>% 
+				sort(decreasing = TRUE) %>% 
+				names %>% 
+				.[show_number]
 			use_data %<>% .[as.character(.[, use_level]) %in% use_taxa, ]
 			use_data[, use_level] %<>% factor(., levels = use_taxa)
 
@@ -769,40 +788,6 @@ trans_network <- R6Class(classname = "trans_network",
 				p <- p + guides(size = guide_legend(title = "Abundance(%)"))
 			}
 			p
-		},
-		sum_link = function(taxa_table, link_table, taxa_level){
-			# first obtain the taxa names
-			all_names <- taxa_table[rownames(taxa_table) %in% unique(c(link_table[,1], link_table[,2])), ] %>%
-				{table(.[, taxa_level])} %>%
-				sort(., decreasing = TRUE) %>% 
-				rownames
-			com_group <- expand.grid(all_names, all_names)
-			colnames(com_group) <- c("C1", "C2")
-			# assign rownames irrespective of the order
-			rownames(com_group) <- apply(com_group, 1, function(x) paste0(x, collapse = "-"))
-			# get the unifrom combined name without regard to the order
-			com_group$uni_name <- apply(com_group, 1, function(x) paste0(sort(x), collapse = "-"))
-			com_group1 <- com_group[, -c(1,2), drop = FALSE]
-			res <- link_table
-			# use taxa name to replace the species name
-			res[, 1] <- taxa_table[res[, 1], taxa_level]
-			res[, 2] <- taxa_table[res[, 2], taxa_level]
-			res$pname <- paste(res[, 1], res[, 2], sep = "-")
-			res %<>% dplyr::group_by(pname) %>% 
-				dplyr::summarise(count = dplyr::n()) %>%
-				as.data.frame(stringsAsFactors = FALSE)
-			res <- dplyr::left_join(res, rownames_to_column(com_group1), by = c("pname" = "rowname")) %>%
-				dplyr::group_by(uni_name) %>% 
-				dplyr::summarise(sum_count = sum(count)) %>%
-				as.data.frame(stringsAsFactors = FALSE)
-			res <- dplyr::left_join(res, com_group, by = c("uni_name" = "uni_name"))
-			res <- reshape2::dcast(res, C1~C2, value.var = "sum_count") %>%
-				`row.names<-`(.[,1]) %>%
-				.[, -1, drop = FALSE] %>%
-				.[all_names, all_names] %>%
-				as.matrix
-			res[is.na(res)] <- 0			
-			res
 		}
 	),
 	lock_class = FALSE,
